// playfield.cpp
#include <QtGui>
#include <iostream>
#include "playfield.h"
#include "constant.h"
using namespace std;
PlayField::PlayField(QWidget *parent): QWidget(parent)
{
  setMinimumSize(GC_fieldWidth, GC_fieldHeight);
  setMaximumSize(GC_fieldWidth, GC_fieldHeight);
  setMouseTracking(true);
  pf_leftPaddleCenterY = GC_paddleHalfHeight;
  pf_rightPaddleCenterY = GC_fieldHeight - GC_paddleHalfHeight;
  pf_ballPosition = TwoDVector(GC_fieldWidth/2, GC_ballRadius * 3, false);
}

void PlayField::paintEvent(QPaintEvent *event)
{
  int ww = width(); // widget's width
  int wh = height(); // widget's height
  // draw background
  QPainter painter(this);
  painter.setBrush(GC_backgroundColor);
  painter.drawRect(0, 0, ww, wh);
  // **************
  // * draw paddles
  painter.setBrush(GC_leftPaddleColor);
  painter.drawChord(GC_leftPaddleCenterX,
		    pf_leftPaddleCenterY -
		    GC_paddleRadius,
		    GC_paddleDiameter,
		    GC_paddleDiameter,
		    GC_leftPaddleStartAngle,
		    GC_paddleSpanAngle);
  painter.setBrush(GC_rightPaddleColor);
  painter.drawChord(GC_rightPaddleCenterX,
		    pf_rightPaddleCenterY -
		    GC_paddleRadius,
		    GC_paddleDiameter,
		    GC_paddleDiameter,
		    GC_rightPaddleStartAngle,
		    GC_paddleSpanAngle);
  // ***********
  // * draw ball
  int bx, by;
  updateBallPosition(bx, by);
  painter.setBrush(GC_ballColor);
  painter.drawEllipse(bx - GC_ballRadius,
		      by - GC_ballRadius,
		      GC_ballDiameter,
		      GC_ballDiameter);
}

void PlayField::keyPressEvent(QKeyEvent * event)
{
  switch (event -> key())
    {
    case Qt::Key_Up:
      break;
    case Qt::Key_Down:
      break;
    default: // do nothing, not a control key
      return;
    }
  // update(); // redraw the playfield if necessary
}

void PlayField::mouseMoveEvent(QMouseEvent * event)
{
  // ignore X since the paddle moves only up and down
  static int lastY = event -> y();
  int newY = event -> y();
  while ((lastY > newY) && (pf_leftPaddleCenterY > GC_paddleHalfHeight))
    {
      // move paddle up but keep the paddle within the playfield
      pf_leftPaddleCenterY --;
      lastY --;
      update(); // redraw the playfield
    }
  while ((lastY < newY) &&
	 (pf_leftPaddleCenterY < (GC_fieldHeight - GC_paddleHalfHeight)))

    {
      pf_leftPaddleCenterY ++;
      lastY ++;
      update(); 
    }
  lastY = newY; // this is necessary because the while block may break
		// before the paddle moves out of the playfield
}
void PlayField::updateBallPosition(int & bx, int & by)
{
  bx = (int) pf_ballPosition.getX();
  by = (int) pf_ballPosition.getY();
  pf_ballPosition.addVector(pf_ballVelocity);
  double newBX = pf_ballPosition.getX();
  double newBY = pf_ballPosition.getY();
  // collision with paddles
  double diffX = newBX + GC_paddleHalfHeight;
  // already shifted right by paddleRadius from the left wall
  double diffY = newBY - pf_leftPaddleCenterY;
  TwoDVector normalVector;
  if ((diffX * diffX + diffY * diffY) <= 
      (GC_ballPaddleRadiusSumSquare + 1)) 
    // + 1 to accommodate float-point imprecision
    {
      // collide with left paddle
      normalVector = TwoDVector(diffX, diffY, true);
      findNewVelocity(normalVector);
      return; // ball cannot collide with a paddle and a wall simultaneously
    }

  // collision with walls
  if (newBX <= GC_ballRadius) // left wall
    {
      normalVector = TwoDVector(1, 0, true);
      findNewVelocity(normalVector);
      return; 
    }
  if (newBX >= (GC_fieldWidth - GC_ballRadius)) // right wall
    {
      normalVector = TwoDVector(-1, 0, true);
      findNewVelocity(normalVector);
      return;
    }
  if (newBY <= GC_ballRadius) // top wall
    {
      normalVector = TwoDVector(0, 1, true);
      findNewVelocity(normalVector);
    }
  if (newBY >= (GC_fieldHeight - GC_ballRadius)) // bottom wall
    {
      normalVector = TwoDVector(0, -1, true);
      findNewVelocity(normalVector);
    }  
}

void PlayField::findNewVelocity(const TwoDVector & normalVector)
{
  TwoDVector newVelocity;
  normalVector.reflect(pf_ballVelocity, newVelocity);
  double vX = newVelocity.getX();
  if ((vX > -0.01) && (vX < 0.01))
    { // almost moving vertically, add some horizontal velocity
      TwoDVector horizontal(vX * 10, 0, false);
      newVelocity.addVector(horizontal);
    }
  pf_ballVelocity = newVelocity;
}
